from functools import wraps
from io import BytesIO as Buffer
from matplotlib import pyplot as plt
from PIL import Image


class Options:
    """Matplotlib image and export options

    https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.savefig.html

    Example:
    ```python
    gif.options.matplotlib["dpi"] = 300
    ```
    """

    def __init__(self):
        self.matplotlib = {}

    def reset(self):
        self.matplotlib = {}


options = Options()


def frame(func):
    """Plot function decorator

    Example:
    ```python
    @gif.frame
    def plot(i):
        plt.scatter(x[:i], y[:i])
        plt.xlim((0, 100))
        plt.ylim((0, 100))
    ```
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        buffer = Buffer()
        func(*args, **kwargs)
        plt.savefig(buffer, format="png", **options.matplotlib)
        plt.close()
        buffer.seek(0)
        img = Image.open(buffer)
        return img

    return wrapper


def save(frames, path, duration=100):
    """Save a collection of frames to a gif

    Arguments:
    - frames (list): Collection of frames built with the @gif.frame decorator
    - path (str): Filename with relative/absolute path (must end with .gif)
    - duration (float): Milliseconds between frames

    Example:
    ```python
    frames = []
    for i in range(10):
        frame = plot(i)
        frames.append(frame)

    gif.save(frames, "test.gif", duration=50)
    ```
    """
    if not path.endswith(".gif"):
        raise ValueError("Path/filename must end with .gif")
    frames[0].save(
        path,
        save_all=True,
        append_images=frames[1:],
        optimize=True,
        duration=duration,
        loop=0,
    )



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="gif",
    version="4.0-rc1",
    description="The matplotlib Animation Extension",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Multimedia :: Graphics",
    ],
    keywords=[
        "gif",
        "gifs",
        "animation",
        "PIL",
        "pillow",
        "matplotlib",
    ],
    url="https://github.com/maxhumber/gif",
    author="Max Humber",
    author_email="max.humber@gmail.com",
    license="MIT",
    py_modules=["gif"],
    install_requires=["matplotlib>=3.5.2", "Pillow>=9.1.0"],
    extras_require={
        "test": [
            "black>=22.3.0",
            "matplotlib>=3.5.2",
            "pandas>=1.4.2",
            "pip>=22.1",
            "pytest>=7.1.2",
        ],
    },
    python_requires=">=3.9",
    setup_requires=["setuptools>=62.1.0"],
)
